﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Runtime.Serialization;
using System.Xml;
using Styx;
using Styx.Combat.CombatRoutine;
using Styx.Helpers;
using Styx.WoWInternals;
using ObjectManager = Styx.WoWInternals.ObjectManager;

namespace katzerle
{
    class MoteExtractorSettings
    {
        public MoteExtractorSettings()
        {
            if (ObjectManager.Me != null)
                try
                {
                    Load();
                }
                catch (Exception e)
                {
                    Logging.Write(System.Drawing.Color.Red, e.Message);
                }
        }

        public bool ArcaneVortex = true;
        public bool WindyCloud = true;
        public bool Felmist = true;
        public bool SwampGas = true;
        public bool SteamCloud = true;
        public bool CinderCloud = true;
        public bool ArcticCloud = true;
        public string BlacklistTimer = "30";
		
		// -------------- Load ConfigFile ---------------
        public void Load()
        {
            //    XmlTextReader reader;
            XmlDocument xml = new XmlDocument();
            XmlNode xvar;

            string sPath = Path.Combine(MotesExtractor.FolderPath, "config\\MoteExtractor.config");

            Logging.WriteDebug("Mote Extractor: Loading config file: {0}", sPath);
            System.IO.FileStream fs = new System.IO.FileStream(@sPath, System.IO.FileMode.OpenOrCreate, System.IO.FileAccess.ReadWrite);
            try
            {
                xml.Load(fs);
                fs.Close();
            }
            catch (Exception e)
            {
                Logging.WriteDebug(e.Message);
                Logging.Write("Mote Extractor: Continuing with Default Config Values");
                fs.Close();
                return;
            }

            //            xml = new XmlDocument();

            try
            {
                //                xml.Load(reader);
                if (xml == null)
                    return;
                // Load Variables
                xvar = xml.SelectSingleNode("//MoteExtractor/ArcaneVortex");
                if (xvar != null)
                {
                    ArcaneVortex = Convert.ToBoolean(xvar.InnerText);
                    Logging.WriteDebug("Mote Extractor Load: " + xvar.Name + "=" + ArcaneVortex.ToString());
                }
                xvar = xml.SelectSingleNode("//MoteExtractor/WindyCloud");
                if (xvar != null)
                {
                    WindyCloud = Convert.ToBoolean(xvar.InnerText);
                    Logging.WriteDebug("Mote Extractor Load: " + xvar.Name + "=" + WindyCloud.ToString());
                }
                xvar = xml.SelectSingleNode("//MoteExtractor/Felmist");
                if (xvar != null)
                {
                    Felmist = Convert.ToBoolean(xvar.InnerText);
                    Logging.WriteDebug("Mote Extractor Load: " + xvar.Name + "=" + Felmist.ToString());
                }
                xvar = xml.SelectSingleNode("//MoteExtractor/SwampGas");
                if (xvar != null)
                {
                    SwampGas = Convert.ToBoolean(xvar.InnerText);
                    Logging.WriteDebug("Mote Extractor Load: " + xvar.Name + "=" + SwampGas.ToString());
                }

                xvar = xml.SelectSingleNode("//MoteExtractor/ArcticCloud");
                if (xvar != null)
                {
                    ArcticCloud = Convert.ToBoolean(xvar.InnerText);
                    Logging.WriteDebug("Mote Extractor Load: " + xvar.Name + "=" + ArcticCloud.ToString());
                }
                xvar = xml.SelectSingleNode("//MoteExtractor/CinderCloud");
                if (xvar != null)
                {
                    CinderCloud = Convert.ToBoolean(xvar.InnerText);
                    Logging.WriteDebug("Mote Extractor Load: " + xvar.Name + "=" + CinderCloud.ToString());
                }
                xvar = xml.SelectSingleNode("//MoteExtractor/SteamCloud");
                if (xvar != null)
                {
                    SteamCloud = Convert.ToBoolean(xvar.InnerText);
                    Logging.WriteDebug("Mote Extractor Load: " + xvar.Name + "=" + SteamCloud.ToString());
                }


                xvar = xml.SelectSingleNode("//MoteExtractor/BlacklistTimer");
                if (xvar != null)
                {
                    BlacklistTimer = Convert.ToString(xvar.InnerText);
                    Logging.WriteDebug("Mote Extractor Load: " + xvar.Name + "=" + BlacklistTimer.ToString());
                }
			}
            catch (Exception e)
            {
                Logging.WriteDebug("Mote Extractor: PROJECTE EXCEPTION, STACK=" + e.StackTrace);
                Logging.WriteDebug("Mote Extractor: PROJECTE EXCEPTION, SRC=" + e.Source);
                Logging.WriteDebug("Mote Extractor: PROJECTE : " + e.Message);
            }
		}
    }
}
